package com.ruoyi.ss.store.service.impl;

import com.ruoyi.common.utils.CollectionUtils;
import com.ruoyi.ss.device.domain.SmDeviceCountVO;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.domain.enums.DeviceGroupBy;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.StoreAssembler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/4/29
 */
@Service
public class StoreAssemblerImpl implements StoreAssembler {

    @Autowired
    private ISmDeviceService deviceService;

    /**
     * 拼接设备数量
     *
     * @param list 店铺列表
     */
    @Override
    public void assembleDeviceCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        Map<Long, Integer> countMap = deviceService.selectCountMapByStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setDeviceCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接营收数据
     *
     * @param list
     */
    @Override
    public void assembleRevenue(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        // 本日收入
        this.assembleTodayIncome(list);
        // 本月收入
        this.assembleMonthIncome(list);
        // 上月收入
        this.assembleLastMonthIncome(list);
        // 在线设备数量
        this.assembleOnlineCount(list);
        // 离线设备数量
        this.assembleOfflineCount(list);
    }

    /**
     * TODO 拼接本月收入
     *
     * @param list
     */
    @Override
    public void assembleMonthIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        for (StoreVo store : list) {
            store.setMonthIncome(BigDecimal.ZERO);
        }
    }

    /**
     * TODO 拼接本日收入
     *
     * @param list
     */
    @Override
    public void assembleTodayIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        for (StoreVo store : list) {
            store.setTodayIncome(BigDecimal.ZERO);
        }
    }

    /**
     * 拼接上月收入
     *
     * @param list
     */
    @Override
    public void assembleLastMonthIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        for (StoreVo store : list) {
            store.setLastMonthIncome(BigDecimal.ZERO);
        }
    }

    /**
     * 拼接在线数量
     *
     * @param list
     */
    @Override
    public void assembleOnlineCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        SmDeviceQuery query = new SmDeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, SmDeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setOnlineCount(count == null ? 0 :count);
        }
    }

    /**
     * 拼接离线数量
     *
     * @param list
     */
    @Override
    public void assembleOfflineCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        SmDeviceQuery query = new SmDeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, SmDeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setOfflineCount(count == null ? 0 :count);
        }
    }
}
